#!/usr/bin/env perl

#---------------------------------------------------------------------
#                          file information
#---------------------------------------------------------------------

# File:     setup/openssh.boot
# Purpose:  Sets up "openssh"
# License:  BSD
# Revision: 210501

#---------------------------------------------------------------------
#                              overview
#---------------------------------------------------------------------

# "/etc/rc.d/rc.local" executes this script at boot time.  This script
# sets up the "openssh" package.  In particular,  it starts the "sshd"
# daemon.

# The  ".boot" filename extension is required.  The  ".boot" extension
# tells the system startup code to execute this script,  as opposed to
# "sourcing" it.

# Note: This script is also used at "build" time.

#---------------------------------------------------------------------
#                            module setup
#---------------------------------------------------------------------

require 5.6.1;
use strict;
use Carp;
use warnings;
                                # Trap warnings
$SIG {__WARN__} = sub { die @_; };

#---------------------------------------------------------------------
#                           basic constants
#---------------------------------------------------------------------
                                
use constant ZERO  => 0;        # Zero
use constant ONE   => 1;        # One

use constant FALSE => 0;        # Boolean FALSE
use constant TRUE  => 1;        # Boolean TRUE
                                
#---------------------------------------------------------------------
#                         program parameters
#---------------------------------------------------------------------

#  $SCRIPT_NAME specifies the name of this  setup program for  message
# purposes.

#  $EXENAME specifies the  target-executable filename,  without a path
# component.

my $SCRIPT_NAME = 'setup/openssh.boot';
my $EXENAME     = 'sshd';

#---------------------------------------------------------------------

#  $INTERR specifies a prefix string used by internal-error messages.

my $INTERR   = "Internal error: $SCRIPT_NAME";

#  $BTMPFILE specifies an absolute pathname for the failed-login "log"
# file used by "sshd".  Note:  The pathname must be  equal  to  '/var/
# log/btmp'.

my $BTMPFILE = '/var/log/btmp';

#---------------------------------------------------------------------

#  $EXEDIR specifies an  absolute path for a standard  "bin" directory
# that contains the target executable  [or a symbolic link to the exe-
# cutable]. Usually, this parameter will be set to '/usr/bin' for ord-
# inary  programs or  '/usr/sbin' for daemons or restricted-use admin-
# istrator programs. However, other directories may be used in various
# cases.

my $EXEDIR  = '/usr/sbin';

#---------------------------------------------------------------------

#  $EXEPATH specifies an absolute pathname  for the  target executable
# [or a symbolic link to the executable].

my $EXEPATH = "$EXEDIR/$EXENAME";

#---------------------------------------------------------------------
#                            main routine
#---------------------------------------------------------------------

sub Main
{
    my $str;                    # Scratch
                                # Full pathname for related directory
    my $VarEmpty = '/var/empty';

#---------------------------------------------------------------------

    my $arg        = shift (@ARGV);
       $arg        = "" unless defined ($arg);
    my $files_only = 0;
       $files_only = 1 if $arg =~ m@^-*files-*only\z@i;

#---------------------------------------------------------------------
# Print a status message.

                                # Setting up process or just files
    if ((-f $EXEPATH) && !$files_only)
    {                           # Process
        print "Setting up $EXENAME\n";
    }
    else
    {                           # Just files
        print "Setting up $EXENAME files\n";
    }

#---------------------------------------------------------------------
# Shut down existing process [if any].

#ifdef NOTDEF
#   system "sureterm $EXENAME";
#endif

#---------------------------------------------------------------------
# Set up special directories and/or files.

    if (!-d $VarEmpty)          # Does $VarEmpty directory exist?
    {                           # No  - Set it up
        system "mkdir -p       $VarEmpty";
        system "chown root:sys $VarEmpty";
    }
                                # Look-up the "sshd" group
    $str = `/usr/bin/grep "^sshd:" /etc/group 2>&1`;
    $str = "" if !defined ($str);

    if (!length ($str))         # Does the "sshd" group exist?
    {                           # No
        system "groupadd sshd"; # Create the group
                                # Create the associated user
        $str = << "END";
useradd -c 'sshd privsep' -d $VarEmpty -g sshd -s /bin/false sshd
END
        system $str;
    }
                                # Set up failed-login "log" file
    system "touch $BTMPFILE" unless -e $BTMPFILE;
    system "chmod 600 $BTMPFILE";

#---------------------------------------------------------------------
# Wrap it up.

    exit (ZERO) if $files_only; # Handle files-only mode
                                # Is this a manual-start network?
    exit (ONE ) if -f '/etc/sysconfig/network-manual';

                                # Disable "sshd" auto-start?
    exit (ONE ) if -f '/etc/sysconfig/disable-auto-sshd';

                                # Confirm that executable exists
    exit (ONE ) unless (-f $EXEPATH) && (-x $EXEPATH);

                                # Wait for network to start
    system "/opt/rjkbin/wait-ipv4.pl";

    system $EXEPATH;            # Start the daemon
    undef;
}

#---------------------------------------------------------------------
#                            main program
#---------------------------------------------------------------------

&Main();                        # Call the main routine
exit ZERO;                      # Normal exit
